@@ -175,7 +175,14 @@ the GCompletion can be found in hashtable
 bfwin->identifier_ac with 
 key Tbflang-context -> value GCompletion
 
+======= Split view / slave widget =======
 
+a slave widget is a widget that does not do scanning itself, it doesn't 
+have a scanning table, it doesn't have any settings. It relies for all
+of these things on the master widget. On a master widget btv->master will
+point to itself, on a slave widget btv->master will point to the master.
+
+The slave widget should always be destroyed before the master. 
 */
 
 #ifndef _BFTEXTVIEW2_H_
@@ -193,7 +200,7 @@ extern void g_none(char * first, ...);
 #endif  
  /**/
 
-/*#define IDENTSTORING*/
+#define IDENTSTORING
 
 #define BF2_OFFSET_UNDEFINED G_MAXUINT32
 
@@ -212,6 +219,7 @@ extern void g_none(char * first, ...);
 #define DBG_BLOCKMATCH DBG_NONE
 #define DBG_TOOLTIP DBG_NONE
 #define DBG_SPELL DBG_NONE
+#define DBG_IDENTIFIER DBG_NONE
 
 #define NUMSCANCHARS 127 /* 128 is ascii, but the last character is never scanned (DEL)
 		and the Ttablerow has one more 16bit value. By setting this to 127 instead of 128
@@ -222,6 +230,9 @@ extern void g_none(char * first, ...);
 /* building the automata and autocompletion cache */
 /*****************************************************************/
 
+#define COMMENT_INDEX_INHERIT 255
+#define COMMENT_INDEX_NONE 254
+
 typedef struct {
 	gboolean autocomplete_case_insens;
 	GCompletion* ac; /* autocompletion items in this context */
@@ -235,6 +246,10 @@ typedef struct {
 					except the characters (symbols) thay may be the begin or end of an identifier such
 					as whitespace, ();[]{}*+-/ etc. */
 	guint8 has_tagclose_from_blockstack; /* this context has xml end patterns that need autoclosing for generix xml tags, based on the tag that is on top of the blockstack */
+	guint8 comment_block; /* block comment index in array scantable->comments 
+				or COMMENT_INDEX_INHERIT (which means inherit) 
+				or COMMENT_INDEX_NONE if there is no block comment  */
+	guint8 comment_line; /* index in array scantable->comments for line comments; see comment_block */
 /*#ifdef HAVE_LIBENCHANT_OLD
 	guint8 needspellcheck;
 #endif / *HAVE_LIBENCHANT*/
@@ -281,14 +296,6 @@ typedef struct {
 	guint16 match;			/* 0 == no match, refers to the index number in array 'matches' */
 } Ttablerow; /* a row in the DFA, right now exactly 256 bytes */
 
-typedef struct {
-	guint8 allsymbols[128]; /* this lookup table holds all symbols for all contexts, and is used to trigger scanning if reduced_scan_triggers is enabled */
-	GArray *table; /* dynamic sized array of Ttablerow: the DFA table */
-	GArray *contexts; /* dynamic sized array of Tcontext that translates a context number into a rownumber in the DFA table */
-	GArray *matches; /* dynamic sized array of Tpattern */
-} Tscantable;
-
-
 typedef enum {
 	comment_type_block,
 	comment_type_line
@@ -300,6 +307,14 @@ typedef struct {
 	Tcomment_type type;
 } Tcomment;
 
+typedef struct {
+	guint8 allsymbols[128]; /* this lookup table holds all symbols for all contexts, and is used to trigger scanning if reduced_scan_triggers is enabled */
+	GArray *table; /* dynamic sized array of Ttablerow: the DFA table, max 65.... entries, we use a guint16 as index */
+	GArray *contexts; /* dynamic sized array of Tcontext that translates a context number into a rownumber in the DFA table */
+	GArray *matches; /* dynamic sized array of Tpattern */
+	GArray *comments; /* Tcomment, has max. 256 entries, we use a guint8 as index */
+} Tscantable;
+
 /*****************************************************************/
 /* scanning the text and caching the results */
 /*****************************************************************/
@@ -308,8 +323,8 @@ typedef struct {
 	guint32 end1_o;
 	guint32 start2_o;
 	guint32 end2_o;
+	guint32 refcount; /* free on 0 */
 	gint16 patternum; /* which pattern (number of the array element in scantable->matches) */
-	guint16 refcount; /* free on 0 */
 	guint8 folded;
 	guint8 foldable; /* FALSE on a single line */
 } Tfoundblock; /* once a start-of-block is found start1 and end1 are set
@@ -325,8 +340,8 @@ typedef struct {
 typedef struct {
 	guint32 start_o;
 	guint32 end_o;
+	guint32 refcount; /* free on 0 */
 	gint16 context; /* number of the element in scantable->contexts */
-	guint16 refcount; /* free on 0 */
 } Tfoundcontext; /* once a start-of-context is found start is set
 						and the Tfoundcontext is added to the GtkTextMark as "context"
 						and the Tfoundcontext is added to the current contextstack.
@@ -366,13 +381,11 @@ typedef struct {
 						we want to remove all tags and want to re-highlight */
 	gchar *filename; /* the .bflang2 file */
 	Tscantable *st; /* NULL or complete */
-	GList *comments; /* NULL or complete */
-	Tcomment *line; /* preferred line comment */
-	Tcomment *block; /* preferred block comment */
 	gchar *smartindentchars;
 	gchar *smartoutdentchars;
 #ifdef HAVE_LIBENCHANT
 	gboolean default_spellcheck;
+	gboolean spell_decode_entities;
 #endif
 	gboolean no_st; /* no scantable, for Text, don't try to load the scantable if st=NULL */
 	gboolean parsing; /* set to TRUE when a thread is parsing the scantable already */
@@ -412,26 +425,41 @@ typedef struct _BluefishTextViewClass BluefishTextViewClass;
 
 struct _BluefishTextView {
 	GtkTextView parent;
-/*	Tscantable *scantable;*/
+	gpointer master; /* points usually to self, but in the case of a slave widget 
+					(two widgets showing the same buffer it will point to the master widget) */
+	gpointer slave; /* usually NULL, but might point to a slave widget */
 	Tbflang *bflang; /* Tbflang */
 	gpointer doc; /* Tdocument */
+	GtkTextBuffer *buffer;
 	GtkTextTag *needscanning;
 #ifdef HAVE_LIBENCHANT
 	GtkTextTag *needspellcheck;
 #endif /*HAVE_LIBENCHANT*/
 	GtkTextTag *blockmatch;
 	Tscancache scancache;
+	
 	guint scanner_idle; /* event ID for the idle function that handles the scanning. 0 if no idle function is running */
 	guint scanner_delayed; /* event ID for the timeout function that handles the delayed scanning. 0 if no timeout function is running */
 	GTimer *user_idle_timer;
 	guint user_idle; /* event ID for the timed function that handles user idle events such as autocompletion popups */
+	guint mark_set_idle; /* event ID for the mark_set idle function that avoids showing matching block bounds while 
+								you hold the arrow key to scroll quickly */
+	gulong insert_text_id;
+	gulong insert_text_after_id;
+	gulong mark_set_id;
+	gulong delete_range_id;
+	gulong delete_range_after_id;
+	
 	gpointer autocomp; /* a Tacwin* with the current autocompletion window */
 	gboolean needs_autocomp; /* a state of the widget, autocomplete is needed on user keyboard actions */
+	gboolean needs_blockmatch; /* a state of the widget, if the cursor position was changed */
 	gboolean key_press_inserted_char; /* FALSE if the key press was used by autocomplete popup, or simply not in our widget */
 	/*gboolean key_press_was_autocomplete;  a state of the widget, if the last keypress was handled by the autocomplete popup window */
 	gboolean showing_blockmatch; /* a state of the widget if we are currently showing a blockmatch */
 	gboolean insert_was_auto_indent; /* a state of the widget if the last keypress (enter) caused 
 													autoindent (so we should unindent on a closing bracket */
+	gboolean needremovetags; /* after we have removed all old highlighting, we set this to FALSE
+										but after a change that needs highlighting we set this to TRUE again */
 
 	/* next three are used for margin painting */
 	gint margin_pixels_per_char;
@@ -460,13 +488,12 @@ struct _BluefishTextViewClass {
 
 GType bluefish_text_view_get_type (void);
 
-GtkWidget * bftextview2_new(void);
-GtkWidget * bftextview2_new_with_buffer(GtkTextBuffer * buffer);
+
 gboolean bluefish_text_view_get_auto_complete(BluefishTextView * btv);
 void bluefish_text_view_set_auto_complete(BluefishTextView * btv, gboolean enable);
 gboolean bluefish_text_view_get_auto_indent(BluefishTextView * btv);
 void bluefish_text_view_set_auto_indent(BluefishTextView * btv, gboolean enable);
-void bftextview2_parse_static_colors(void);
+void bftextview2_init_globals(void);
 void bluefish_text_view_set_colors(BluefishTextView * btv, gchar * const *colors);
 void bluefish_text_view_set_mimetype(BluefishTextView * btv, const gchar *mime);
 gboolean bluefish_text_view_get_show_blocks(BluefishTextView * btv);
@@ -486,7 +513,7 @@ void bluefish_text_view_scan_cleanup(BluefishTextView * btv);
 void bluefish_text_view_rescan(BluefishTextView * btv);
 void bftextview2_schedule_scanning(BluefishTextView * btv);
 gboolean bluefish_text_view_in_comment(BluefishTextView * btv, GtkTextIter *its, GtkTextIter *ite);
-
+Tcomment *bluefish_text_view_get_comment(BluefishTextView *btv, GtkTextIter *it, Tcomment_type preferred_type);
 void bluefish_text_view_multiset(BluefishTextView *btv
 			, gpointer doc
 			, gint view_line_numbers
@@ -495,5 +522,8 @@ void bluefish_text_view_multiset(BluefishTextView *btv
 			, gint autocomplete
 			, gint show_mbhl);
 
+GtkWidget * bftextview2_new(void);
+GtkWidget * bftextview2_new_with_buffer(GtkTextBuffer * buffer);
+GtkWidget *bftextview2_new_slave(BluefishTextView *master);
 
 #endif
