@@ -284,7 +284,7 @@ bluefish_encodings_dialog_create(GType type,
 	g_signal_connect(dialog, "response", G_CALLBACK(encodings_dialog_response_lcb), dialog);
 
 	hbox = gtk_hbox_new(FALSE, 0);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), hbox, TRUE, TRUE, 8);
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog))), hbox, TRUE, TRUE, 8);
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_box_pack_start(GTK_BOX(hbox), vbox, TRUE, TRUE, 8);
