@@ -1,7 +1,7 @@
 /* Bluefish HTML Editor
  * filefilter.c - 
  *
- * Copyright (C) 2002,2003,2004,2005,2006,2007 Olivier Sessink
+ * Copyright (C) 2002-2010 Olivier Sessink
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include "file.h"
 #include "file_dialogs.h"
 #include "gtk_easy.h"        /* destroy_disposable_menu_cb() */
-#include "stringlist.h"      /* count_array() */
+#include "stringlist.h"      /* free_stringlist() */
 
 typedef struct {
 	gchar *pattern;
