@@ -29,7 +29,7 @@
 #include "bf_lib.h"
 #include "pixmap.h"
 #include "gui.h" /* statusbar_message() */
-#include "stringlist.h" /* count_array() */
+/* #include "stringlist.h"  count_array() */
 
 #ifdef WIN32
 #define DIRSTR "\\"
@@ -80,7 +80,7 @@ GtkTreeModel *treemodel_from_arraylist(GList *list, gint numcols) {
 	
 	for (tmplist=g_list_first(list);tmplist;tmplist=tmplist->next){
 		gchar **arr = (gchar **)tmplist->data;
-		if (count_array(arr) >= numcols-1) {
+		if (g_strv_length(arr) >= numcols-1) {
 			gtk_list_store_append(GTK_LIST_STORE(retm),&iter);
 			for (i=0;i<numcols-1;i++) {
 				g_print("set column %d to value %s\n",i,arr[i]);
@@ -184,9 +184,9 @@ void setup_toggle_item(GtkItemFactory * ifactory, gchar * path, gint state) {
 		g_print("Cannot set-up menu widget %s\n",path);
 		return;
 	}
-	if ((GTK_CHECK_MENU_ITEM(toggle)->active) != state) {
+	if ((gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(toggle))) != state) {
 		/*gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(toggle), state);*/
-		GTK_CHECK_MENU_ITEM(toggle)->active = state;
+		gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(toggle),state);
 	}
 }
 
@@ -204,11 +204,14 @@ void button_apply(gchar ** config_var, GtkWidget * entry)
 {
 	const gchar *tmpstring;
 	DEBUG_MSG("button_apply, start\n");
+	
+	if (!config_var || !entry)
+		return;
 
 	tmpstring = gtk_button_get_label(GTK_BUTTON(entry));
 	DEBUG_MSG("button_apply, tmpstring(%p)=%s\n", tmpstring, tmpstring);
 	if (tmpstring) {
-		if (config_var != NULL) {
+		if (*config_var != NULL) {
 			g_free(*config_var);
 		}
 		*config_var = g_strdup(tmpstring);  /* copy */ 
@@ -232,14 +235,25 @@ void button_apply(gchar ** config_var, GtkWidget * entry)
 void string_apply(gchar ** config_var, GtkWidget * widget)
 {
 	gchar *tmpstring;
-	if (GTK_IS_COMBO_BOX(widget)) {
+
+	if (!config_var || !widget)
+		return;	
+	
+	if (GTK_IS_FONT_BUTTON(widget)) {
+		tmpstring = g_strdup(gtk_font_button_get_font_name(GTK_FONT_BUTTON(widget)));
+	} else if (GTK_IS_COLOR_BUTTON(widget)) {
+		GdkColor color;
+
+		gtk_color_button_get_color(GTK_COLOR_BUTTON(widget), &color);
+		tmpstring = gdk_color_to_string(&color);
+	} else if (GTK_IS_COMBO_BOX(widget)) {
 		tmpstring = gtk_combo_box_get_active_text(GTK_COMBO_BOX(widget));
 	} else {
 		tmpstring = gtk_editable_get_chars(GTK_EDITABLE(widget), 0, -1);
 	}
 	DEBUG_MSG("string_apply, tmpstring(%p)=%s\n", tmpstring, tmpstring);
 	if (tmpstring) {
-		if (config_var != NULL) {
+		if (*config_var != NULL) {
 			g_free(*config_var);
 		}
 		*config_var = tmpstring;
@@ -262,13 +276,24 @@ void string_apply(gchar ** config_var, GtkWidget * widget)
  */
 void integer_apply(gint *config_var, GtkWidget * widget, gboolean is_checkbox) {
 	if (is_checkbox) {
-		*config_var = (GTK_TOGGLE_BUTTON(widget)->active);
+		*config_var = (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget)));
 	} else {
 		*config_var = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(widget));
 	}
 	DEBUG_MSG("integer_apply, config_var(%p)=%i\n", config_var, *config_var);
 }
 
+static void boxed_widget(const gchar *labeltext, GtkWidget *widget, GtkWidget *box) {
+	GtkWidget *hbox, *label;
+
+	hbox = gtk_hbox_new(FALSE,3);
+	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
+	label = gtk_label_new_with_mnemonic(labeltext);
+	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
+	gtk_box_pack_start(GTK_BOX(hbox), widget, FALSE, FALSE, 3);
+	gtk_label_set_mnemonic_widget(GTK_LABEL(label), widget);
+}
+
 GtkWidget *combobox_with_popdown_sized(const gchar * setstring, GList * which_list, gint editable, gint width) {
 	GtkWidget *returnwidget;
 	
@@ -368,15 +393,9 @@ GtkWidget *boxed_entry_with_text(const gchar * setstring, gint max_lenght, GtkWi
  * Return value: #GtkWidget* pointer to the new entry widget
  */
 GtkWidget *boxed_full_entry(const gchar * labeltext, gchar * setstring,gint max_lenght, GtkWidget * box) {
-	GtkWidget *hbox, *return_widget, *label;
-
-	hbox = gtk_hbox_new(FALSE,3);
-	label = gtk_label_new_with_mnemonic(labeltext);
-	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 3);
-	return_widget = boxed_entry_with_text(setstring, max_lenght, hbox);
-	gtk_label_set_mnemonic_widget(GTK_LABEL(label), return_widget);
-	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
-
+	GtkWidget *return_widget;
+	return_widget = entry_with_text(setstring, max_lenght);
+	boxed_widget(labeltext, return_widget, box);
 	return return_widget;
 }
 /**
@@ -555,13 +574,10 @@ GtkWidget *optionmenu_with_value(gchar **options, gint curval) {
 
 GtkWidget *boxed_optionmenu_with_value(const gchar *labeltext, gint curval, GtkWidget *box, gchar **options) {
 	GtkWidget *returnwidget;
-	GtkWidget *hbox;
-
-	hbox = gtk_hbox_new(FALSE,3);
-	gtk_box_pack_start(GTK_BOX(box), hbox, FALSE, FALSE, 3);
-	gtk_box_pack_start(GTK_BOX(hbox), gtk_label_new(labeltext), FALSE, FALSE, 3);
+	
 	returnwidget = optionmenu_with_value(options, curval);
-	gtk_box_pack_start(GTK_BOX(hbox), returnwidget, FALSE, FALSE, 3);
+	boxed_widget(labeltext, returnwidget, box);
+	
 	return returnwidget;
 }
 
@@ -711,7 +727,7 @@ gchar *textbuffer_get_all_chars(GtkTextBuffer *buffer) {
  * Return value: #GtkWidget* to this_widget
  */
 GtkWidget *apply_font_style(GtkWidget * this_widget, gchar * fontstring) {
-	if (fontstring) {
+	if (fontstring && fontstring[0] != '\0') {
 		PangoFontDescription *font_desc;
 		font_desc = pango_font_description_from_string(fontstring);
 		gtk_widget_modify_font(this_widget, font_desc);
@@ -760,7 +776,7 @@ GtkWidget *bf_allbuttons_backend(const gchar *label, gboolean w_mnemonic, gint b
 			gtk_container_add(GTK_CONTAINER(button), new_pixmap(bf_pixmaptype));
 		}
 	}
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default(button, TRUE);
 	g_signal_connect(G_OBJECT(button), "clicked", func, func_data);
 	return button;
 }
@@ -825,7 +841,7 @@ GtkWidget *bf_allbuttons_backend(const gchar *label, gboolean w_mnemonic, gint b
  */
 GtkWidget *bf_gtkstock_button(const gchar * stock_id, GCallback func, gpointer func_data) {
 	GtkWidget *button = gtk_button_new_from_stock(stock_id);
-	GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
+	gtk_widget_set_can_default(button,TRUE);
 	g_signal_connect(G_OBJECT(button), "clicked", func, func_data);
 	return button;
 }
@@ -936,7 +952,7 @@ Tmultientrywidget *build_multi_entry_window(gchar *title,GCallback ok_func
 	gint arrlen,i;
 	GtkWidget *table,*hbox,*but;
 
-	arrlen = count_array(labelarr);
+	arrlen = g_strv_length(labelarr);
 	if (arrlen >10) arrlen = 10;
 	Tmultientrywidget *mew = g_new0(Tmultientrywidget,1);
 	mew->data = data;
@@ -963,6 +979,8 @@ Tmultientrywidget *build_multi_entry_window(gchar *title,GCallback ok_func
 	gtk_widget_show_all(mew->win);
 }
 #endif
+
+#ifdef NEED_HIG_DIALOG
 /**************************************************************************/
 /***********************  BUTTON DIALOG FUNCTIONS  ************************/
 /**************************************************************************/
@@ -987,7 +1005,7 @@ static void hig_dialog_backend (GtkDialog *dialog, gchar *primary, gchar *second
 	gtk_window_set_resizable (GTK_WINDOW (dialog), FALSE);
 	gtk_dialog_set_has_separator (GTK_DIALOG (dialog), FALSE);
 
-	vbox = GTK_DIALOG (dialog)->vbox;
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(dialog));
 	
 	hbox = gtk_hbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (hbox), 12);
@@ -1021,7 +1039,8 @@ static void hig_dialog_backend (GtkDialog *dialog, gchar *primary, gchar *second
 	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
 	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
 }
-
+#endif /*NEED_HIG_DIALOG*/
+#ifdef NEED_PROGRESSBAR
 /************************************************************************/
 /*********************** PROGRESS-BAR FUNCTIONS *************************/
 /************************************************************************/
@@ -1122,7 +1141,7 @@ gpointer progress_popup(GtkWidget *win,gchar *title, guint maxvalue) {
 	/* Label, if applicable. Append pretty icon! */
 	hig_dialog_backend(GTK_DIALOG (p->owner), title, NULL, GTK_STOCK_DIALOG_INFO);
 
-	gtk_box_pack_start (GTK_BOX (GTK_DIALOG (p->owner)->vbox),
+	gtk_box_pack_start (GTK_BOX (gtk_dialog_get_content_area(GTK_DIALOG(p->owner))),
                         p->bar, TRUE, TRUE, 12);
 
 	p->timer = gtk_timeout_add (500, progress_update, p);
@@ -1130,6 +1149,7 @@ gpointer progress_popup(GtkWidget *win,gchar *title, guint maxvalue) {
 	
 	return (gpointer) p;
 }
+#endif /* NEED_PROGRESSBAR */
 
 /************************************************************************/
 /************************ file_but_* FUNCTIONS **************************/
@@ -1148,12 +1168,12 @@ static void file_but_clicked_lcb(GtkWidget * widget, Tfilebut *fb) {
 	setfile = gtk_editable_get_chars(GTK_EDITABLE(GTK_ENTRY(fb->entry)),0,-1);
 	/* if setfile is empty we should probably use the current document basedir ? right? */
 	if (!setfile || strlen(setfile)==0) {
-		if (fb->bfwin && fb->bfwin->current_document->uri) {
+		if (fb->bfwin && fb->bfwin->current_document && fb->bfwin->current_document->uri) {
 			if (setfile) g_free(setfile);
 			setfile = g_file_get_uri(fb->bfwin->current_document->uri);
 		} else
 				setfile = NULL;
-	} else if (setfile && setfile[0] != '/' && strchr(setfile, ':')==NULL && fb->bfwin && fb->bfwin->current_document->uri) {
+	} else if (setfile && setfile[0] != '/' && strchr(setfile, ':')==NULL && fb->bfwin && fb->bfwin->current_document && fb->bfwin->current_document->uri) {
 		/* if setfile is a relative name, we should try to make it a full path. relative names
 		cannot start with a slash or with a scheme (such as file://)
 		 */
@@ -1188,7 +1208,7 @@ static void file_but_clicked_lcb(GtkWidget * widget, Tfilebut *fb) {
 	g_free(setfile);
 	DEBUG_MSG("file_but_clicked_lcb, return_file returned %s\n",tmpstring);
 	if (tmpstring) {
-		if (!fb->fullpath && fb->bfwin) {
+		if (!fb->fullpath && fb->bfwin && fb->bfwin->current_document ) {
 			if (fb->bfwin->current_document->uri != NULL) {
 				gchar *doc_curi;
 				/* the function g_file_get_relative_path cannot create links that don't 
@@ -1292,7 +1312,7 @@ static gboolean file_chooser_custom_filter_func(GtkFileFilterInfo *filter_info,g
 	gint ret;
 	Tfchooser_filter *cf = data;
 	if (!filter_info->display_name || filter_info->display_name[0] == '\0') return FALSE; /* error condition ?? */
-	if (!GTK_TOGGLE_BUTTON(cf->show_backup)->active) {
+	if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(cf->show_backup))) {
 		gint namelen = strlen(filter_info->display_name);
 		if (filter_info->display_name[namelen-1] == '~' ) return FALSE;
 	}
@@ -1459,7 +1479,7 @@ GtkWidget * file_chooser_dialog(Tbfwin *bfwin, const gchar *title, GtkFileChoose
 /************************************************************************/
 
 static void ungroupradoiitems(GtkWidget *menu) {
-	GList *tmplist = g_list_first(GTK_MENU_SHELL(menu)->children);
+	GList *tmplist = g_list_first(gtk_container_get_children(GTK_CONTAINER(menu)));
 	while (tmplist) {
 		GtkWidget *sub;
 		DEBUG_MSG("ungroupradiomenuitems, another item\n");
@@ -1531,19 +1551,19 @@ gchar *ask_accelerator_dialog(const gchar *title) {
 	gtk_window_set_type_hint(GTK_WINDOW(dialog1), GDK_WINDOW_TYPE_HINT_DIALOG);
 	gtk_dialog_set_has_separator(GTK_DIALOG(dialog1), FALSE);
 	
-	label1 = gtk_label_new("<b>Keystroke choice </b>");
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog1)->vbox), label1, FALSE, FALSE, 0);
+	label1 = gtk_label_new(_("<b>Keystroke choice </b>"));
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog1))), label1, FALSE, FALSE, 0);
 	gtk_label_set_use_markup(GTK_LABEL(label1), TRUE);
 	gtk_label_set_justify(GTK_LABEL(label1), GTK_JUSTIFY_CENTER);
 	gtk_misc_set_padding(GTK_MISC(label1), 2, 2);
 	
-	label2 = gtk_label_new("\nPress requested key combination.\nPlease use Ctrl, Shift, Alt key with any other key.\n<i>Esc to cancel, Delete to remove the accelerator.</i>");
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog1)->vbox), label2, FALSE, FALSE, 0);
+	label2 = gtk_label_new(_("\nPress requested key combination.\nPlease use Ctrl, Shift, Alt key with any other key.\n<i>Esc to cancel, Delete to remove the accelerator.</i>"));
+	gtk_box_pack_start(GTK_BOX(gtk_dialog_get_content_area(GTK_DIALOG(dialog1))), label2, FALSE, FALSE, 0);
 	gtk_label_set_use_markup(GTK_LABEL(label2), TRUE);
 	gtk_label_set_justify(GTK_LABEL(label2), GTK_JUSTIFY_CENTER);
 	gtk_misc_set_padding(GTK_MISC(label2), 2, 2);
 	
-	gtk_button_box_set_layout(GTK_BUTTON_BOX(GTK_DIALOG(dialog1)->action_area), GTK_BUTTONBOX_END);
+	gtk_button_box_set_layout(GTK_BUTTON_BOX(gtk_dialog_get_action_area(GTK_DIALOG(dialog1))), GTK_BUTTONBOX_END);
 	g_signal_connect(G_OBJECT(dialog1),"key-press-event",G_CALLBACK(accelerator_key_press_lcb),dialog1);
 	
 	gtk_widget_show_all(dialog1);
@@ -1557,6 +1577,29 @@ gchar *ask_accelerator_dialog(const gchar *title) {
 	return retval;
 }
 
+static void accelerator_button_clicked_lcb(GtkWidget *widget, gpointer data) {
+	gchar *tmpstr;
+	tmpstr = ask_accelerator_dialog(_("Press shortcut key combination"));
+	if (tmpstr && tmpstr[0] != '\0') {
+		gtk_button_set_label(GTK_BUTTON(widget), tmpstr);
+	}
+	g_free(tmpstr);
+}
+
+GtkWidget *accelerator_button(const gchar *accel) {
+	GtkWidget *retval;
+	
+	retval = gtk_button_new_with_label(accel);
+	g_signal_connect(G_OBJECT(retval), "clicked", G_CALLBACK(accelerator_button_clicked_lcb), retval);
+	return retval;
+}
+GtkWidget *boxed_accelerator_button(const gchar *labeltext, const gchar *accel, GtkWidget *box) {
+	GtkWidget *returnwidget;
+	returnwidget = accelerator_button(accel);
+	boxed_widget(labeltext, returnwidget, box);
+	return returnwidget;
+}
+
 gchar *gdk_color_to_hexstring(GdkColor *color, gboolean websafe) {
 	gchar *tmpstr;
 	tmpstr = g_malloc(8*sizeof(char));
