@@ -31,13 +31,16 @@
 #include "bluefish.h"
 
 #ifdef MAC_INTEGRATION
-#include <ige-mac-integration.h>
+/*#include <ige-mac-integration.h>*/
+#include <gtkosxapplication.h>
 #endif
 
 #include "bf_lib.h"			/* append_string_to_file() */
 #include "bookmark.h"
 #include "dialog_utils.h"
 #include "document.h"
+#include "doc_comments.h"
+#include "doc_text_tools.h"
 #include "encodings_dialog.h"
 #include "file_dialogs.h"
 #include "gtk_easy.h"            /* window_full, bf_stock_ok_button */
@@ -81,7 +84,8 @@ static void menu_file_operations_cb(Tbfwin *bfwin,guint callback_action, GtkWidg
 		file_open_advanced_cb(NULL,bfwin);
 	break;
 	case 4:
-		doc_reload(bfwin->current_document, NULL, TRUE);
+		if (bfwin->current_document)
+			doc_reload(bfwin->current_document, NULL, TRUE);
 	break;
 	case 5:
 		file_save_cb(NULL, bfwin);
@@ -173,6 +177,9 @@ static void menu_file_operations_cb(Tbfwin *bfwin,guint callback_action, GtkWidg
 	case 36:
 		convert_to_columns(CURDOC(bfwin));
 	break;
+	case 37:
+		rewrap_lines(CURDOC(bfwin));
+	break;
 #ifdef HAVE_PYTHON
 	case 99:
 		{
@@ -287,6 +294,7 @@ static GtkItemFactoryEntry menu_items[] = {
 	{N_("/Document/_Visible Spacing"), NULL, doc_menu_lcb, 6, "<ToggleItem>"},
 	{N_("/Document/Show Right Margin"), NULL, doc_menu_lcb, 14, "<ToggleItem>"},
 	{N_("/Document/Highlight block delimiters"), NULL, doc_menu_lcb, 15, "<ToggleItem>"},
+	{N_("/Document/Split view"), NULL, doc_menu_lcb, 17, "<ToggleItem>"},
 #ifdef HAVE_LIBENCHANT
 	{N_("/Document/_Spell Check"), NULL, doc_menu_lcb, 13, "<ToggleItem>"},
 #endif
@@ -331,12 +339,12 @@ static GtkItemFactoryEntry menu_items[] = {
 	{N_("/Go/L_ast Document"), NULL, gui_notebook_switch, 4, "<StockItem>", GTK_STOCK_GOTO_LAST},
 	{"/Go/sep2", NULL, NULL, 0, "<Separator>"},
 	{N_("/Go/F_irst Bookmark"), NULL, bookmark_menu_cb, 1, "<StockItem>", GTK_STOCK_GOTO_TOP},
-	{N_("/Go/P_revious Bookmark"), NULL, bookmark_menu_cb, 2, "<StockItem>", GTK_STOCK_GO_UP},
-	{N_("/Go/N_ext Bookmark"), NULL, bookmark_menu_cb, 3, "<StockItem>", GTK_STOCK_GO_DOWN},
+	{N_("/Go/P_revious Bookmark"), "<shift><control>j", bookmark_menu_cb, 2, "<StockItem>", GTK_STOCK_GO_UP},
+	{N_("/Go/N_ext Bookmark"), "<shift><control>k", bookmark_menu_cb, 3, "<StockItem>", GTK_STOCK_GO_DOWN},
 	{N_("/Go/Last _Bookmark"), NULL, bookmark_menu_cb, 4, "<StockItem>", GTK_STOCK_GOTO_BOTTOM},
 	{"/Go/sep3", NULL, NULL, 0, "<Separator>"},
 	{N_("/Go/Goto _Line"), "<control>l", gui_gotoline_frame_show, 1, "<StockItem>", GTK_STOCK_JUMP_TO},
-	{N_("/Go/Goto line number in _selection"), NULL, go_to_line_from_selection_cb, 1, "<Item>"},
+	{N_("/Go/Goto line number in _selection"), "<shift><control>l", go_to_line_from_selection_cb, 1, "<Item>"},
 	{N_("/Go/Jump to reference"), "<control>j", doc_menu_lcb, 16, "<Item>"},
 	{N_("/_Project"), NULL, NULL, 0, "<Branch>"},
 	{"/Project/tearoff1", NULL, NULL, 0, "<Tearoff>"},
@@ -365,8 +373,9 @@ static GtkItemFactoryEntry menu_items[] = {
 	{N_("/Tools/_Word Count"), NULL, word_count_cb, 1, "<Item>"},
 	{"/Tools/sep2", NULL, NULL, 0, "<Separator>"},
 	{N_("/Tools/Strip T_railing Whitespace"), NULL, menu_file_operations_cb, 29, "<Item>"},
-	{N_("/Tools/_Join lines"), NULL, menu_file_operations_cb, 33, "<Item>"},
-	{N_("/Tools/Sp_lit lines"), NULL, menu_file_operations_cb, 34, "<Item>"},
+	{N_("/Tools/_Join lines together"), NULL, menu_file_operations_cb, 33, "<Item>"},
+	{N_("/Tools/Sp_lit lines on right margin"), NULL, menu_file_operations_cb, 34, "<Item>"},
+	{N_("/Tools/Rewrap _lines"), NULL, menu_file_operations_cb, 37, "<Item>"},
 	{N_("/Tools/Indenting To T_abs"), NULL, menu_file_operations_cb, 30, "<Item>"},
 	{N_("/Tools/Indenting To S_paces"), NULL, menu_file_operations_cb, 31, "<Item>"},
 	{N_("/Tools/Merge Lines Into Col_umns"), NULL, menu_file_operations_cb, 36, "<Item>"},
@@ -433,8 +442,8 @@ static GtkWidget *remove_menuitem_in_list_by_label(const gchar *labelstring, GLi
 	tmplist = g_list_first(*menuitemlist);
 	while (tmplist) {
 		DEBUG_MSG("remove_recent_entry, tmplist=%p, data=%p\n", tmplist, tmplist->data);
-		DEBUG_MSG("remove_recent_entry, tmplist->data=%s\n",GTK_LABEL(GTK_BIN(tmplist->data)->child)->label);
-		if(!strcmp(GTK_LABEL(GTK_BIN(tmplist->data)->child)->label, labelstring)) {
+		DEBUG_MSG("remove_recent_entry, tmplist->data=%s\n",GTK_LABEL(gtk_bin_get_child(GTK_BIN(tmplist->data)))->label);
+		if(!strcmp(gtk_label_get_label(GTK_LABEL(gtk_bin_get_child(GTK_BIN(tmplist->data)))), labelstring)) {
 			tmp = tmplist->data;
 			*menuitemlist = g_list_remove(*menuitemlist, tmplist->data);
 			DEBUG_MSG("remove_recent_entry, returning %p\n", tmp);
@@ -495,7 +504,7 @@ static void create_parent_and_tearoff(gchar *menupath, GtkItemFactory *ifactory)
 
 static void menu_current_document_type_change(GtkMenuItem *menuitem,Tbfw_dynmenu *bdm) {
 	DEBUG_MSG("menu_current_document_type_change, started for bflang %p\n", bdm->data);
-	if (GTK_CHECK_MENU_ITEM(menuitem)->active) {
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem)) && bdm->bfwin->current_document) {
 		doc_set_mimetype(bdm->bfwin->current_document, ((Tbflang *)bdm->data)->mimetypes->data);
 	}
 	DEBUG_MSG("menu_current_document_type_change, finished\n");
@@ -546,9 +555,6 @@ void template_menu_rebuild(Tbfwin *bfwin,GtkItemFactory *item_factory) {
 		}
 		tmplist = g_list_previous(tmplist);
 	}
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
 }
 
 void filetype_menu_rebuild(Tbfwin *bfwin,GtkItemFactory *item_factory) {
@@ -576,10 +582,6 @@ void filetype_menu_rebuild(Tbfwin *bfwin,GtkItemFactory *item_factory) {
 		tmplist = g_list_previous(tmplist);
 	}
 	g_list_free(list);
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
-
 }
 
 /*
@@ -605,7 +607,6 @@ void menu_create_main(Tbfwin *bfwin, GtkWidget *vbox) {
 	gtk_accel_map_add_entry("<bluefishmain>/Go/Next Document", GDK_Page_Down, GDK_CONTROL_MASK);
 	gtk_accel_map_add_entry("<bluefishmain>/Go/First Document", GDK_Page_Up, GDK_SHIFT_MASK | GDK_CONTROL_MASK);
 	gtk_accel_map_add_entry("<bluefishmain>/Go/Last Document", GDK_Page_Down, GDK_SHIFT_MASK | GDK_CONTROL_MASK);
-	gtk_widget_show(bfwin->menubar);
 	setup_toggle_item(item_factory, "/View/Main Toolbar", bfwin->session->view_main_toolbar);
 	setup_toggle_item(item_factory, "/View/Side Pane", bfwin->session->view_left_panel);
 	setup_toggle_item(item_factory, "/View/Statusbar", bfwin->session->view_statusbar);
@@ -643,30 +644,34 @@ static GtkWidget *create_recent_entry(Tbfwin *bfwin, const gchar *filename, gboo
 /*******************************************************************/
 
 static GtkWidget *remove_recent_entry(Tbfwin *bfwin, const gchar *filename, gboolean is_project) {
-	GList *tmplist;
-	GList **worklist;
-	gpointer tmp;
+	if (filename)
+	{
+		GList *tmplist;
+		GList **worklist;
+		gpointer tmp;
 
-	worklist = (is_project) ? &bfwin->menu_recent_projects : &bfwin->menu_recent_files;
+		worklist = (is_project) ? &bfwin->menu_recent_projects : &bfwin->menu_recent_files;
 
-	if(strcmp(filename, "last") ==0) {
-		tmplist = g_list_first(*worklist);
-		if (tmplist) {
-			tmp = tmplist->data;
-			DEBUG_MSG("remove_recent_entry, remove last entry\n");
-			*worklist = g_list_remove(*worklist, tmplist->data);
-			return tmp;
+		if(strcmp(filename, "last") ==0) {
+			tmplist = g_list_first(*worklist);
+			if (tmplist) {
+				tmp = tmplist->data;
+				DEBUG_MSG("remove_recent_entry, remove last entry\n");
+				*worklist = g_list_remove(*worklist, tmplist->data);
+				return tmp;
+			} else {
+				DEBUG_MSG("remove_recent_entry, worklist contained no items, returning NULL\n");
+				return NULL;
+			}
 		} else {
-			DEBUG_MSG("remove_recent_entry, worklist contained no items, returning NULL\n");
-			return NULL;
+			return remove_menuitem_in_list_by_label(filename, worklist);
 		}
-	}	else {
-		return remove_menuitem_in_list_by_label(filename, worklist);
-	}
+	} else
+		return NULL;
 }
 
 static void open_recent_project_cb(GtkWidget *widget, Tbfwin *bfwin) {
-	GFile *uri = g_file_new_for_uri(GTK_LABEL(GTK_BIN(widget)->child)->label);
+	GFile *uri = g_file_new_for_uri(gtk_label_get_label(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))));
 	if (uri) {
 		project_open_from_file(bfwin, uri);
 		add_to_recent_list(bfwin,uri, 0, TRUE);
@@ -678,7 +683,7 @@ static void open_recent_project_cb(GtkWidget *widget, Tbfwin *bfwin) {
  * This function should be called when a menu from the Open Recent list
  * has been selected. */
 static void open_recent_file_cb(GtkWidget *widget, Tbfwin *bfwin) {
-	GFile *uri = g_file_new_for_uri(GTK_LABEL(GTK_BIN(widget)->child)->label);
+	GFile *uri = g_file_new_for_uri(gtk_label_get_label(GTK_LABEL(gtk_bin_get_child(GTK_BIN(widget)))));
 	if (uri) {
 		doc_new_from_uri(bfwin, uri, NULL, FALSE, FALSE, -1, -1);
 		add_to_recent_list(bfwin,uri, 0, FALSE);
@@ -718,9 +723,6 @@ GList *recent_menu_from_list(Tbfwin *bfwin, GList *startat, gboolean is_project)
 		retlist = g_list_append(retlist, create_recent_entry(bfwin,tmplist->data,is_project,FALSE));
 		tmplist = g_list_next(tmplist);
 	}
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
 	return retlist;
 }
 
@@ -766,41 +768,41 @@ static void register_recent(gchar *curi, gboolean is_project) {
  * menu bar, and (if nessecary) deletes the last entry */
 void add_to_recent_list(Tbfwin *bfwin, GFile *file, gint closed_file, gboolean is_project) {
 	gchar *filename = g_file_get_uri(file);
-	register_recent(filename,is_project);
-	if (closed_file) {
-		GList *tmplist = g_list_first(main_v->bfwinlist);
-		while (tmplist) {
-			Tbfwin *curbfwin = BFWIN(tmplist->data);
-			if (!curbfwin->project || curbfwin == bfwin || is_project) {
-				GtkWidget *tmp;
-				GList **worklist;
-				worklist = (is_project) ? &curbfwin->menu_recent_projects : &curbfwin->menu_recent_files;
+	if (filename)
+	{
+		register_recent(filename,is_project);
+		if (closed_file) {
+			GList *tmplist = g_list_first(main_v->bfwinlist);
+			while (tmplist) {
+				Tbfwin *curbfwin = BFWIN(tmplist->data);
+				if (!curbfwin->project || curbfwin == bfwin || is_project) {
+					GtkWidget *tmp;
+					GList **worklist;
+					worklist = (is_project) ? &curbfwin->menu_recent_projects : &curbfwin->menu_recent_files;
 
-				/* First of all, create the entry and insert it at the list*/
-				*worklist = g_list_append(*worklist,create_recent_entry(curbfwin,filename,is_project,TRUE));
+					/* First of all, create the entry and insert it at the list*/
+					*worklist = g_list_append(*worklist,create_recent_entry(curbfwin,filename,is_project,TRUE));
 
-				DEBUG_MSG("add_to_recent_list, inserted item in menu\n");
-				if(g_list_length(*worklist) > main_v->props.max_recent_files) {
-					tmp = remove_recent_entry(bfwin,"last",is_project);
-					if (tmp) {
-						DEBUG_MSG("add_to_recent_list, list too long, entry %s to be deleted\n", GTK_LABEL(GTK_BIN(tmp)->child)->label);
-						gtk_widget_hide(tmp);
-						gtk_widget_destroy(tmp);
+					DEBUG_MSG("add_to_recent_list, inserted item in menu\n");
+					if(g_list_length(*worklist) > main_v->props.max_recent_files) {
+						tmp = remove_recent_entry(bfwin,"last",is_project);
+						if (tmp) {
+							DEBUG_MSG("add_to_recent_list, list too long, entry %s to be deleted\n", GTK_LABEL(GTK_BIN(tmp)->child)->label);
+							gtk_widget_hide(tmp);
+							gtk_widget_destroy(tmp);
+						}
 					}
 				}
+				tmplist = g_list_next(tmplist);
 			}
-			tmplist = g_list_next(tmplist);
 		}
+		if (is_project) {
+			main_v->globses.recent_projects = add_to_history_stringlist(main_v->globses.recent_projects, filename, FALSE,TRUE);
+		} else {
+			bfwin->session->recent_files = add_to_history_stringlist(bfwin->session->recent_files, filename, FALSE,TRUE);
+		}
+		g_free(filename);
 	}
-	if (is_project) {
-		main_v->globses.recent_projects = add_to_history_stringlist(main_v->globses.recent_projects, filename, FALSE,TRUE);
-	} else {
-		bfwin->session->recent_files = add_to_history_stringlist(bfwin->session->recent_files, filename, FALSE,TRUE);
-	}
-	g_free(filename);
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
 }
 
 /*****************/
@@ -839,7 +841,7 @@ static void external_filter_dialog_response_lcb(GtkWidget *widget,gint response_
 	gint begin=0,end=-1;
 	gtk_widget_destroy(widget);
 	doc_restore_selection(fd->selsave, TRUE); /* the restore will also free the Tselectionsave */
-	if (response_id == 1) {
+	if (response_id == 1 && fd->bdm->bfwin->current_document) {
 		doc_get_selection(fd->bdm->bfwin->current_document,&begin,&end);
 	}
 	filter_command(fd->bdm->bfwin, arr[1],begin,end);
@@ -852,7 +854,7 @@ static void external_filter_lcb(GtkWidget *widget, Tbfw_dynmenu *bdm) {
 	/* if we have a selection, and the filter can be used on a selection,
 	 we should ask if it should be the complete file or the selection */
 
-	if (operatable_on_selection(arr[1]) && (doc_has_selection(bdm->bfwin->current_document))) {
+	if (operatable_on_selection(arr[1]) && (bdm->bfwin->current_document && doc_has_selection(bdm->bfwin->current_document))) {
 		GtkWidget *dialog, *but;
 		Tfilterdialog *fd;
 		fd = g_slice_new(Tfilterdialog);
@@ -904,7 +906,7 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 		/*  arr[0] = name
 		 *  arr[1] = command
 		 */
-		if (count_array(arr)==2) {
+		if (g_strv_length(arr)==2) {
 			Tbfw_dynmenu *bdm = g_slice_new(Tbfw_dynmenu);
 			bdm->bfwin = bfwin;
 			bdm->data = arr;
@@ -914,7 +916,7 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 			DEBUG_MSG("external_menu_rebuild,creating,bfwin=%p,bdm=%p,menuitem=%p\n",bfwin,bdm,bdm->menuitem);
 			bfwin->menu_external = g_list_append(bfwin->menu_external, bdm);
 		} else {
-			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_filter; array count =%d\n",count_array(arr));
+			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_filter; array count =%d\n",g_strv_length(arr));
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -926,7 +928,7 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 		 *  arr[1] = command
 		 *  arr[2] = is_default_browser
 		 */
-		if (count_array(arr)==3) {
+		if (g_strv_length(arr)==3) {
 			Tbfw_dynmenu *bdm = g_slice_new(Tbfw_dynmenu);
 			bdm->bfwin = bfwin;
 			bdm->data = arr;
@@ -938,7 +940,7 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 			}
 			bfwin->menu_external = g_list_append(bfwin->menu_external, bdm);
 		} else {
-			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_command; array count =%d\n",count_array(arr));
+			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_command; array count =%d\n",g_strv_length(arr));
 		}
 		tmplist = g_list_next(tmplist);
 	}
@@ -957,7 +959,7 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 		 * arr[5] = command
 		 * arr[6] = show_all_output
 		 */
-		if (count_array(arr)==6) {
+		if (g_strv_length(arr)==6) {
 			Tbfw_dynmenu *bdm = g_slice_new(Tbfw_dynmenu);
 			bdm->data = arr;
 			bdm->bfwin = bfwin;
@@ -966,21 +968,18 @@ void external_menu_rebuild(Tbfwin *bfwin) {
 			bdm->menuitem = create_dynamic_menuitem(bfwin,"/Tools/Outputbox",arr[0],G_CALLBACK(menu_outputbox_lcb),(gpointer)bdm,-1);
 			bfwin->menu_outputbox = g_list_append(bfwin->menu_outputbox,bdm);
 		} else {
-			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_outputbox; array count =%d\n",count_array(arr));
+			DEBUG_MSG("external_menu_rebuild, CORRUPT ENTRY IN external_outputbox; array count =%d\n",g_strv_length(arr));
 		}
 		tmplist = g_list_next(tmplist);
 	}
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
 }
 
 static void menu_current_document_encoding_change(GtkMenuItem *menuitem,Tbfw_dynmenu *bdm) {
-	if (GTK_CHECK_MENU_ITEM(menuitem)->active) {
+	if (gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(menuitem))) {
 		gchar *encoding = (gchar *)bdm->data;
 		Tbfwin *bfwin = bdm->bfwin;
 		DEBUG_MSG("menu_current_document_encoding_change, encoding=%s\n",encoding);
-		if (encoding) {
+		if (encoding && bfwin->current_document) {
 			if ((!bfwin->current_document->encoding || strcmp(encoding,bfwin->current_document->encoding)!=0)) {
 				if (bfwin->current_document->encoding) g_free(bfwin->current_document->encoding);
 				bfwin->current_document->encoding = g_strdup(encoding);
@@ -1008,7 +1007,7 @@ void encoding_menu_rebuild(Tbfwin *bfwin) {
 	parent_menu = gtk_item_factory_get_widget(gtk_item_factory_from_widget(bfwin->menubar), N_("/Document/Character Encoding"));
 	while (tmplist) {
 		gchar **strarr = (gchar **)tmplist->data;
-		if (count_array(strarr)==3 && strarr[2][0]=='1') {
+		if (g_strv_length(strarr)==3 && strarr[2][0]=='1') {
 			Tbfw_dynmenu *bdm = g_slice_new(Tbfw_dynmenu);
 			gchar *label = g_strdup_printf ("%s (%s)", strarr[0], strarr[1]);
 			/*g_print("add encoding %s:%s:%s to the menu\n",strarr[0],strarr[1],strarr[2]);*/
@@ -1025,14 +1024,11 @@ void encoding_menu_rebuild(Tbfwin *bfwin) {
 		}
 		tmplist = g_list_previous(tmplist);
 	}
-/*#ifdef MAC_INTEGRATION
-	ige_mac_menu_sync(GTK_MENU_SHELL(BFWIN(bfwin)->menubar));
-#endif*/
 }
 
 void menu_current_document_set_toggle_wo_activate(Tbfwin *bfwin, gpointer filetype, gchar *encoding) {
 	Tbfw_dynmenu *bdm = find_bfw_dynmenu_by_data_in_list(bfwin->menu_filetypes, filetype);
-	if (bdm && filetype && bdm->menuitem && !GTK_CHECK_MENU_ITEM(bdm->menuitem)->active) {
+	if (bdm && filetype && bdm->menuitem && !gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(bdm->menuitem))) {
 		g_signal_handler_disconnect(G_OBJECT(bdm->menuitem),bdm->signal_id);
 		gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(bdm->menuitem), TRUE);
 		bdm->signal_id = g_signal_connect(G_OBJECT(bdm->menuitem), "activate",G_CALLBACK(menu_current_document_type_change), (gpointer) bdm);
