@@ -31,169 +31,27 @@
 #include "../undo_redo.h"
 #include "../dialog_utils.h"
 #include "../rcfile.h"
-
+#include "../xml_entity.h"
 typedef struct {
 	gint convert_num;
-	gint convert_iso;
-	gint convert_symbol;
-	gint convert_special;
-	gint convert_xml;
-	gint IE_apos_workaround;
+gint convert_iso;
+gint convert_symbol;
+gint convert_special;
+gint convert_xml;
+gint IE_apos_workaround;
 } Tentitysetting;
 
 typedef struct {
-	Tentitysetting c2e;
-	Tentitysetting e2c;
+Tentitysetting c2e;
+Tentitysetting e2c;
 } Tentitiessession;
 
 typedef struct {
-	GHashTable* lookup;
+GHashTable* lookup;
 } Tentities;
 
 Tentities entities_v;
 
-/* see http://www.w3.org/TR/xhtml1/dtds.html for more information about the defined entities */
-
-static gchar *entities_iso8859_1[] = {
-	/* although nsbp is part of this set, we choose not to convert it */
-	"iexcl", "cent", "pound", "curren", "yen", "brvbar", "sect", "uml", "copy", "ordf",
-		"laquo", "not", "shy", "reg", "macr", "deg", "plusmn", "sup2", "sup3", "acute", "micro",
-		"para", "middot", "cedil", "sup1", "ordm", "raquo", "frac14", "frac12", "frac34", "iquest",
-		"Agrave", "Aacute", "Acirc", "Atilde", "Auml", "Aring", "AElig", "Ccedil", "Egrave",
-		"Eacute", "Ecirc", "Euml", "Igrave", "Iacute", "Icirc", "Iuml", "ETH", "Ntilde", "Ograve",
-		"Oacute", "Ocirc", "Otilde", "Ouml", "times", "Oslash", "Ugrave", "Uacute", "Ucirc", "Uuml",
-		"Yacute", "THORN", "szlig", "agrave", "aacute", "acirc", "atilde", "auml", "aring", "aelig",
-		"ccedil", "egrave", "eacute", "ecirc", "euml", "igrave", "iacute", "icirc", "iuml", "eth",
-		"ntilde", "ograve", "oacute", "ocirc", "otilde", "ouml", "divide", "oslash", "ugrave",
-		"uacute", "ucirc", "uuml", "yacute", "thorn", "yuml", NULL
-};
-static gchar *entities_symbols[] = {
-	"fnof", "Alpha", "Beta", "Gamma", "Delta", "Epsilon", "Zeta", "Eta", "Theta", "Iota", "Kappa",
-		"Lambda", "Mu", "Nu", "Xi", "Omicron", "Pi", "Rho", "Sigma", "Tau", "Uplsilon", "Phi",
-		"Chi", "Psi", "Omega", "alpha", "beta", "gamma", "delta", "epsilon", "zeta", "eta", "theta",
-		"iota", "kappa", "lambda", "mu", "nu", "xi", "omicron", "pi", "rho", "sigmaf", "sigma",
-		"tau", "upsilon", "phi", "chi", "psi", "omega", "thetasym", "upsih", "piv", "bull",
-		"hellip", "prime", "Prime", "oline", "frasl", "weierp", "image", "real", "trade", "alefsym",
-		"larr", "uarr", "rarr", "darr", "harr", "crarr", "lArr", "uArr", "rArr", "dArr", "hArr",
-		"forall", "part", "exist", "empty", "nabla", "isin" "notin", "ni", "prod", "sum", "minus",
-		"lowast", "radic", "prop", "infin", "ang", "and", "or", "cap", "cup", "int", "there4",
-		"sim", "cong", "asymp", "ne", "equiv", "le", "ge", "sub", "sup", "nsub", "sube", "supe",
-		"oplus", "otimes", "perp", "sdot", "lceil", "rceil", "lfloor", "rfloor", "lang", "rang",
-		"loz", "spades", "clubs", "hearts", "diams", NULL
-};
-static gchar *entities_special[] = {
-	"OElig", "oelig", "Scaron", "scaron", "Yuml", "circ", "tilde",
-		"ensp", "emsp", "thinsp", "zwnj", "zwj", "lrm", "rlm", "ndash", "mdash", "lsquo", "rsquo",
-		"sbquo", "ldquo", "rdquo", "bdquo", "dagger", "Dagger", "permil", "lsaquo", "rsaquo",
-		"euro", NULL
-};
-static gchar *entities_xml[] = {
-	"nbsp", "quot", "amp", "lt", "gt", "apos", NULL
-};
-/*
-* the unicode characters for iso8859_1 are 161 + the index in the array
-*/
-static gint16 entity_unicode_symbols[] = {
-	402, 913, 914, 915, 916, 917, 918, 919, 920, 921, 922, 923, 924, 925, 926, 927, 928, 929, 931,
-		932, 933, 934, 935, 936, 937, 945, 946, 947, 948, 949, 950, 951, 952, 953, 954, 955, 956,
-		957, 958, 959, 960, 961, 962, 963, 964, 965, 966, 967, 968, 977, 978, 982, 8226, 8230, 8242,
-		8443, 8254, 8260, 8472, 8465, 8476, 8482, 8501, 8592, 8593, 8594, 8595, 8596, 8629, 8656,
-		8657, 8658, 8659, 8660, 8704, 8706, 8707, 8709, 8711, 8712, 8713, 8715, 8719, 8721, 8722,
-		8727, 8730, 8733, 8734, 8736, 8743, 8744, 8745, 8746, 8747, 8756, 8764, 8773, 8776, 8800,
-		8801, 8804, 8805, 8834, 8835, 8836, 8838, 8839, 8853, 8855, 8869, 8901, 8968, 8969, 8970,
-		8971, 9001, 9002, 9674, 9824, 9827, 9829, 9830, -1
-};
-static gint16 entity_unicode_special[] = {
-	338, 339, 352, 353, 376, 710, 732, 8194, 8195, 8201, 8204, 8205, 8206, 8207,
-		8211, 8212, 8216, 8217, 8218, 8220, 8221, 8222, 8224, 8225, 8240, 8249, 8250, 8364, -1
-};
-static gint16 entity_unicode_xml[] = {
-	160, '"', '&', '<', '>', '\'', -1
-};
-/* returns -1 if index not found */
-static gint16 index_in_array(gchar **arr,gchar *string) {
-	gint i;
-	for (i=0;arr[i]!=NULL;i++) {
-		if (strcmp(arr[i],string)==0) {
-			return i;
-		}
-	}
-	return -1;
-}
-
-static gint index_in_array2(gint16 *arr, guint val) {
-	gint i;
-	for (i=0;arr[i]!=-1;i++) {
-		if (arr[i]==val) {
-			return i;
-		}
-	}
-	return -1;
-}
-
-gunichar unichar_for_entity(gchar *entity,gboolean numerical, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml) {
-	gint16 indx;
-	if (!entity) return -1;
-	if (entity[0] == '#') {
-		if (numerical) {
-			/* convert the remaining characters to a number */
-			if (entity[1] == 'x') {
-				/* from hexadecimal */
-				return g_ascii_strtoull(&entity[2],NULL,16);
-			} else {
-				/* from decimal */
-				return g_ascii_strtoull(&entity[1],NULL,10);
-			}
-		}
-		return -1;
-	}
-	if (iso8859_1) {
-		indx = index_in_array(entities_iso8859_1, entity);
-		if (indx != -1) {
-			return indx + 161;
-		}
-	}
-	if (symbols) {
-		indx = index_in_array(entities_symbols, entity);
-		if (indx != -1) {
-			return entity_unicode_symbols[indx];
-		}
-	}
-	if (specials) {
-		indx = index_in_array(entities_special, entity);
-		if (indx != -1) {
-			return entity_unicode_special[indx];
-		}
-	}
-	if (xml) {
-		indx = index_in_array(entities_xml, entity);
-		if (indx != -1) {
-			return entity_unicode_xml[indx];
-		}
-	}
-	return -1;
-}
-
-gchar *entity_for_unichar(gunichar uchar, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml) {
-	gint16 indx;
-	if (iso8859_1 && uchar >= 161 && uchar < 255) {
-		return entities_iso8859_1[uchar - 161];
-	}
-	if (symbols && (uchar == 402 || (uchar >= 913 && uchar <= 982) ||(uchar >= 8226 && uchar <= 9830))) {
-		indx = index_in_array2(entity_unicode_symbols, uchar);
-		if (indx != -1) return entities_symbols[indx];
-	}
-	if (specials && ((uchar >= 338 && uchar <= 376) || uchar == 710 || uchar == 732 || (uchar >= 8194 && uchar <= 8364) )) {
-		indx = index_in_array2(entity_unicode_special, uchar);
-		if (indx != -1) return entities_special[indx];
-	}
-	if (xml) {
-		indx = index_in_array2(entity_unicode_xml, uchar);
-		if (indx != -1) return entities_xml[indx];
-	}
-	return NULL;
-}
-
 typedef enum {
 	mode_urlencode,
 	mode_urldecode,
@@ -229,54 +87,6 @@ static void doc_code_selection(Tdocument *doc, Treplace_mode mode) {
 	}
 }
 
-/* inbuf is a NULL terminated UTF8 string */
-gchar *entities_to_utf8(const gchar *inbuf) {
-	const gchar *found, *prevfound;
-	gchar *outbuf, *outbufloc;
-
-	outbuf = g_malloc0(strlen(inbuf));
-	prevfound = inbuf;
-	outbufloc = outbuf;
-	found = g_utf8_strchr(inbuf,-1, '&');
-	while (found) {
-		gchar *endfound;
-    
-		endfound = g_utf8_strchr(found,-1, ';');
-		if (endfound && endfound - found <= 7) {
-			gchar *entity, tmp[7];
-			gint len;
-			gunichar unic;
-      
-			entity = g_strndup(found+1, (endfound-found)-1);
-			len = (found - prevfound);
-			memcpy(outbufloc,prevfound,len);
-			outbufloc += len;
-			unic = unichar_for_entity(entity, TRUE,TRUE,TRUE,TRUE,FALSE);
-			if (unic != -1) {
-				DEBUG_MSG("entities_to_utf8, unic=%d for entity '%s'\n",unic,entity);
-				memset(tmp, 0, 7);
-				g_unichar_to_utf8(unic, tmp);
-				len = strlen(tmp);
-				DEBUG_MSG("entities_to_utf8, tmp='%s', len=%d\n",tmp,len);
-				memcpy(outbufloc, tmp, len);
-				outbufloc += len;
-			} else {
-				len = endfound - found+1;
-				memcpy(outbufloc, found, len);
-				outbufloc += len;
-			}
-			g_free(entity);
-      
-			prevfound = g_utf8_next_char(endfound);
-			found = g_utf8_strchr(prevfound,-1, '&');
-		} else {
-			found = g_utf8_strchr(g_utf8_next_char(found),-1, '&');
-		}
-	}
-	memcpy(outbufloc,prevfound,strlen(prevfound)+1);
-	return outbuf;
-}
-
 void doc_entities_to_utf8(Tdocument *doc, gint start, gint end, gboolean numerical, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml) {
 	gchar *buf;
 	const gchar *found, *prevfound;
@@ -294,7 +104,8 @@ void doc_entities_to_utf8(Tdocument *doc, gint start, gint end, gboolean numeric
 			gunichar unic;
       
 			entity = g_strndup(found+1, (endfound-found)-1);
-			unic = unichar_for_entity(entity,numerical,iso8859_1,symbols,specials,xml);
+			/*unic = unichar_for_entity(entity,numerical,iso8859_1,symbols,specials,xml);*/
+			unic = xmlentity2unichar(entity,numerical, iso8859_1, symbols, specials, xml);
 			if (unic != -1) {
 				guint cfound,cendfound;
 				gchar tmp[7];
@@ -317,61 +128,28 @@ void doc_entities_to_utf8(Tdocument *doc, gint start, gint end, gboolean numeric
 	}
 }
 
-
-gchar *utf8_to_entities(const gchar *inbuf, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml) {
-	gchar *outbuf;
-	const gchar *srcp = inbuf;
-	gunichar unichar = g_utf8_get_char(inbuf);
-	/* we use a lot of memory for this function, but it makes the code simple */
-	outbuf = g_malloc0(8 * strlen(inbuf)*sizeof(gchar));
-	while (unichar) {
-		gchar *entity;
-		entity = entity_for_unichar(unichar, iso8859_1, symbols, specials, xml);
-		if (entity) {
-			outbuf = strncat(outbuf, "&", 1);
-			outbuf = strncat(outbuf, entity, 7);
-			outbuf = strncat(outbuf, ";", 1);
-			srcp = g_utf8_next_char(srcp);
-		} else {
-			gchar *nxt = g_utf8_next_char(srcp);
-			outbuf = strncat(outbuf, srcp, (nxt-srcp));
-			srcp = nxt;
-		}
-		unichar = g_utf8_get_char (srcp);
-	}
-	return outbuf;
-}
-
-void doc_utf8_to_entities(Tdocument *doc, gint start, gint end, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml, gboolean IE_apos_workaround) {
+void doc_utf8_to_entities(Tdocument *doc, gint start, gint end, gboolean iso8859_1, gboolean symbols, gboolean specials, gboolean xml, gboolean numerical, gboolean IE_apos_workaround) {
 	gunichar unichar;
-	gchar *buf, *srcp, *workaround=NULL;
+	gchar *buf, *srcp;
 	guint docpos=start;
 	buf = doc_get_chars(doc,start,end);
 	srcp = buf;
 
-	if (xml && IE_apos_workaround) {
-		workaround = entities_xml[5];
-		entities_xml[5] = "#39";
-	}	
-	
 	unichar = g_utf8_get_char(buf);
 	while (unichar) {
 		gchar *entity;
-		entity = entity_for_unichar(unichar, iso8859_1, symbols, specials, xml);
+		/*entity = entity_for_unichar(unichar, iso8859_1, symbols, specials, xml);*/
+		entity = unichar2xmlentity(unichar, iso8859_1, symbols, specials, xml, numerical, IE_apos_workaround);
 		if (entity) {
-			gchar *replacew = g_strconcat("&", entity, ";", NULL);
-			doc_replace_text_backend(doc,replacew,docpos,docpos+1);
-			docpos += (strlen(replacew)-1);
-			g_free(replacew);
+			doc_replace_text_backend(doc,entity,docpos,docpos+1);
+			docpos += (strlen(entity)-1);
+			g_free(entity);
 		}
 		srcp = g_utf8_next_char(srcp);
 		unichar = g_utf8_get_char (srcp);
 		docpos++;
 	}
 	g_free(buf);
-	if (workaround) {
-		entities_xml[5] = workaround;
-	}
 }
 
 typedef enum {
@@ -397,25 +175,25 @@ static void ew_response_lcb(GtkDialog * dialog, gint response, Tentwin * ew) {
 	if (response == GTK_RESPONSE_ACCEPT) {
 		gint start=0, end=-1;
 		gint scope = gtk_combo_box_get_active(GTK_COMBO_BOX(ew->scope));
-			if (ew->numerical)
-				ew->eset->convert_num = GTK_TOGGLE_BUTTON(ew->numerical)->active;
-		ew->eset->convert_iso = GTK_TOGGLE_BUTTON(ew->iso8859_1)->active;
-		ew->eset->convert_symbol = GTK_TOGGLE_BUTTON(ew->symbol)->active;
-		ew->eset->convert_special = GTK_TOGGLE_BUTTON(ew->special)->active;
-		ew->eset->convert_xml = GTK_TOGGLE_BUTTON(ew->xml)->active;
+		if (ew->numerical)
+			ew->eset->convert_num = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->numerical));
+		ew->eset->convert_iso = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->iso8859_1));
+		ew->eset->convert_symbol = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->symbol));
+		ew->eset->convert_special = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->special));
+		ew->eset->convert_xml = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->xml));
 		if (ew->IE_apos_workaround)
-			ew->eset->IE_apos_workaround = GTK_TOGGLE_BUTTON(ew->IE_apos_workaround)->active;
+			ew->eset->IE_apos_workaround = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->IE_apos_workaround));
     
 		if (scope == 0 || (scope == 1 && doc_get_selection(ew->bfwin->current_document, &start, &end))) {
 			doc_unre_new_group_action_id(ew->bfwin->current_document,0);
 			if (ew->mode == mode_char2ent) {
 				doc_utf8_to_entities(ew->bfwin->current_document, start, end,  
-						GTK_TOGGLE_BUTTON(ew->iso8859_1)->active, GTK_TOGGLE_BUTTON(ew->symbol)->active, 
-						GTK_TOGGLE_BUTTON(ew->special)->active, GTK_TOGGLE_BUTTON(ew->xml)->active, ew->eset->IE_apos_workaround);
+						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->iso8859_1)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->symbol)), 
+						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->special)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->xml)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->numerical)), ew->eset->IE_apos_workaround);
 			} else {
-				doc_entities_to_utf8(ew->bfwin->current_document, start, end, GTK_TOGGLE_BUTTON(ew->numerical)->active,
-						GTK_TOGGLE_BUTTON(ew->iso8859_1)->active, GTK_TOGGLE_BUTTON(ew->symbol)->active, 
-						GTK_TOGGLE_BUTTON(ew->special)->active, GTK_TOGGLE_BUTTON(ew->xml)->active);
+				doc_entities_to_utf8(ew->bfwin->current_document, start, end, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->numerical)),
+						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->iso8859_1)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->symbol)),
+						gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->special)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->xml)));
 			}
 			doc_unre_new_group_action_id(ew->bfwin->current_document,0);
 		} else /* (scope == 2) */ {
@@ -425,12 +203,12 @@ static void ew_response_lcb(GtkDialog * dialog, gint response, Tentwin * ew) {
 				doc_unre_new_group_action_id(DOCUMENT(tmplist->data), unre_action_id);
 				if (ew->mode == mode_char2ent) {
 					doc_utf8_to_entities(DOCUMENT(tmplist->data), 0, -1, 
-							GTK_TOGGLE_BUTTON(ew->iso8859_1)->active, GTK_TOGGLE_BUTTON(ew->symbol)->active, 
-							GTK_TOGGLE_BUTTON(ew->special)->active, GTK_TOGGLE_BUTTON(ew->xml)->active, ew->eset->IE_apos_workaround);
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->iso8859_1)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->symbol)), 
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->special)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->xml)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->numerical)), ew->eset->IE_apos_workaround);
 				} else {
-					doc_entities_to_utf8(DOCUMENT(tmplist->data), 0, -1, GTK_TOGGLE_BUTTON(ew->numerical)->active,
-							GTK_TOGGLE_BUTTON(ew->iso8859_1)->active, GTK_TOGGLE_BUTTON(ew->symbol)->active, 
-							GTK_TOGGLE_BUTTON(ew->special)->active, GTK_TOGGLE_BUTTON(ew->xml)->active);
+					doc_entities_to_utf8(DOCUMENT(tmplist->data), 0, -1, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->numerical)),
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->iso8859_1)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->symbol)),
+							gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->special)), gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(ew->xml)));
 				}
 				doc_unre_new_group_action_id(DOCUMENT(tmplist->data),0);
 			}
@@ -457,9 +235,11 @@ static void entity_dialog(Tbfwin *bfwin, Tentmode mode, Tentitysetting *eset) {
 	g_signal_connect(G_OBJECT(ew->dialog), "response", G_CALLBACK(ew_response_lcb), ew);
 	window_delete_on_escape(GTK_WINDOW(ew->dialog));
 
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(ew->dialog));
+
 	alignment = gtk_alignment_new(0, 0, 1, 1);
 	gtk_alignment_set_padding(GTK_ALIGNMENT(alignment), 12, 12, 6, 6);
-	gtk_box_pack_start(GTK_BOX(GTK_DIALOG(ew->dialog)->vbox), alignment, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), alignment, TRUE, TRUE, 0);
 
 	vbox = gtk_vbox_new(FALSE, 6);
 	gtk_container_add(GTK_CONTAINER(alignment), vbox);
@@ -474,13 +254,9 @@ static void entity_dialog(Tbfwin *bfwin, Tentmode mode, Tentitysetting *eset) {
 	dialog_box_label_new(_("Sco_pe:"), 0.5, 0.5, hbox, 0);
 	gtk_box_pack_start(GTK_BOX(hbox), ew->scope, FALSE, FALSE, 0);
   
-	if (mode == mode_ent2char) {
-		ew->numerical = gtk_check_button_new_with_mnemonic(_("Convert numerical encoded characters &#99;"));
-		gtk_box_pack_start(GTK_BOX(vbox), ew->numerical, FALSE, FALSE, 0);
-		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ew->numerical),eset->convert_num);
-	} else {
-		ew->numerical = NULL;
-	}
+	ew->numerical = gtk_check_button_new_with_mnemonic(_("Convert numerical encoded characters &#99;"));
+	gtk_box_pack_start(GTK_BOX(vbox), ew->numerical, FALSE, FALSE, 0);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ew->numerical),eset->convert_num);
 
 	ew->iso8859_1 = gtk_check_button_new_with_mnemonic(_("Convert _iso-8859-1 characters"));
 	gtk_box_pack_start(GTK_BOX(vbox), ew->iso8859_1, FALSE, FALSE, 0);
