@@ -99,9 +99,9 @@ static void snippets_insert_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_
 	gchar *before=NULL, *after=NULL;
 	xmlNodePtr cur;
 	int i=0;
-	GtkWidget *table, *label;
+	GtkWidget *table, *label, *vbox;
 	gchar *tmpstr;
-	gint beforelen,afterlen;
+	gint beforelen=0,afterlen=0;
 	
 	title = xmlGetProp(leaf, (const xmlChar *)"title");
 	sid = g_new0(Tsnippet_insert_dialog,1);
@@ -113,7 +113,8 @@ static void snippets_insert_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_
 							NULL);
 	xmlFree(title);
 	gtk_dialog_set_default_response (GTK_DIALOG(sid->dialog),GTK_RESPONSE_ACCEPT);
-	gtk_box_set_spacing(GTK_BOX(GTK_DIALOG(sid->dialog)->vbox),6);
+	vbox = gtk_dialog_get_content_area(GTK_DIALOG(sid->dialog));
+	gtk_box_set_spacing(GTK_BOX(vbox),6);
 	table = gtk_table_new(num_vars+1, 3, FALSE);
 	gtk_table_set_col_spacings(GTK_TABLE(table), 12);
 	gtk_table_set_row_spacings(GTK_TABLE(table), 6);
@@ -153,7 +154,7 @@ static void snippets_insert_dialog(Tsnippetswin *snw, xmlNodePtr leaf, gint num_
 	gtk_table_attach(GTK_TABLE (table), label, 0, 2, 0, 1, GTK_FILL, GTK_FILL, 0, 0);
 	
 	sid->textentry[i] = NULL;
-	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(sid->dialog)->vbox),table);
+	gtk_container_add(GTK_CONTAINER(vbox),table);
 	
 	gtk_widget_show_all(sid->dialog);	
 	if (gtk_dialog_run(GTK_DIALOG(sid->dialog))==GTK_RESPONSE_ACCEPT) {
